/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTJob_h__
#define __RTJob_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

// When more than one thread of control exists, we need to identify stages
// in the life of a thread, so that interactions between jobs may be adjusted
// according to their phases. The class RTJob embodies this abstraction.
// When a thread is created, it is given an instance of a concrete
// subclass of RTJob to execute. The new thread signals the completion of
// its initialization by moving to the running phase. When the job is
// done, the phase becomes finished which should terminate mainLoop.
// The thread destructor invokes the kill method to coordinate shutdown.

// Example:
//
//   RTJob    * job    = new RTSpecialJob; // some class derived from RTJob
//   RTThread * thread = new RTThread( job, stack_size, priority );
//
//   ... // other interactions with job
//
//   delete thread;
//   delete job;

class RTMutex;

class RTJob
{
public:
    enum Phase
    {
	created,    // initialization
	running,    // most of the time will be spent in this state
	finished    // job finished and thread gone
    };

    // these are intended for use from "outside" the job
			    RTJob( void );
    virtual                 ~RTJob( void );

    // these can be used by anyone
    virtual const char    * name( void ) const = 0;
    RTS_INLINE Phase        phase( void ) const;

    // the thread responsible for the job will invoke this function
    virtual void            mainLoop( void ) = 0;

    // the thread destructor uses this function
    virtual void            kill( void ) = 0;

private:
    Phase                   _phase;

protected:
#if USE_THREADS
    RTMutex               * _mutex;
#endif

    RTS_INLINE void         enter( Phase );

private:
    // unavailable methods
    RTJob( const RTJob & );
    RTJob & operator=( const RTJob & );
};

#if RTS_INLINES
#include <RTJob.inl>
#endif

#endif // __RTJob_h__
